import { Page } from "@/pages/base";
import { Management } from "@/management/main";
import { Box, Button, CircularProgress, Container, IconButton, Tooltip, Typography } from "@material-ui/core";
import React, { useEffect, useState } from "react";
import { Trans, useTranslation } from "react-i18next";
import { useAsyncState } from "@/hooks";
import api from "@/management/api";
import { Async } from "@/components/async";
import MaterialTable, { Action, Column } from "material-table";
import { default as StaticPage } from "@/data/page";
import { Delete, FileFind, Pencil, Refresh } from "mdi-material-ui";
import { encapsulate, one, OneOrMany } from "@/helpers";
import { Actions } from "@/components";
import { useSpacing } from "@/styles";
import { useHistory } from "react-router-dom";
import { Add, Edit } from "@material-ui/icons";
import { createPortal } from "react-dom";
import { EditStaticPageDialog } from "@/management/page/edit";
import { Confirm } from "@/components/confirm";
import useTheme from "@material-ui/core/styles/useTheme";
import { BulkActions } from "@/management/common/BulkActions";
import { MaterialTableTitle } from "@/management/common/MaterialTableTitle";
import { actionsColumn, fieldComparator, multilingualStringComparator } from "@/management/common/helpers";
import { createDeleteAction } from "@/management/common/DeleteResourceAction";
import { MultilingualCell } from "@/management/common/MultilangualCell";

const label = (page: StaticPage) => page.title.pl;

export const StaticPageManagement = () => {
    const { t } = useTranslation("management");
    const [ result, setPagesPromise ] = useAsyncState<StaticPage[]>();
    const [ selected, setSelected ] = useState<StaticPage[]>([]);
    const spacing = useSpacing(2);

    const updatePageList = () => {
        setPagesPromise(api.page.all());
    }

    useEffect(updatePageList, []);

    const EditStaticPageAction = ({ page }: { page: StaticPage }) => {
        const [ open, setOpen ] = useState<boolean>(false);

        const handlePageCreation = async (page: StaticPage) => {
            await api.page.save(page);
            setOpen(false);
            updatePageList();
        }

        return <>
            <Tooltip title={ t("actions.edit") as any }>
                <IconButton onClick={ () => setOpen(true) }><Edit /></IconButton>
            </Tooltip>
            { open && createPortal(
                <EditStaticPageDialog open={ open } onSave={ handlePageCreation } page={ page } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element
            ) }
        </>
    }

    const CreateStaticPageAction = () => {
        const [ open, setOpen ] = useState<boolean>(false);

        const handlePageCreation = async (page: StaticPage) => {
            await api.page.save(page);
            setOpen(false);
            updatePageList();
        }

        return <>
            <Button variant="contained" color="primary" startIcon={ <Add /> } onClick={ () => setOpen(true) }>{ t("create") }</Button>
            { createPortal(
                <EditStaticPageDialog open={ open } onSave={ handlePageCreation } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element
            ) }
        </>
    }

    const handlePageDeletion = async (page: OneOrMany<StaticPage>) => {
        await api.page.remove(page);
        updatePageList();
    }

    const DeleteStaticPageAction = createDeleteAction<StaticPage>({ label, onDelete: handlePageDeletion })

    const PreviewStaticPageAction = ({ page }: { page: StaticPage }) => {
        const history = useHistory();
        const handlePagePreview = async () => history.push(`/${page.slug}`);

        return <Tooltip title={ t("actions.preview") as string }>
            <IconButton onClick={ handlePagePreview }><FileFind /></IconButton>
        </Tooltip>;
    }

    const columns: Column<StaticPage>[] = [
        {
            render: page => <MultilingualCell value={ page.title }/>,
            title: t("page.field.title"),
            customSort: fieldComparator("title", multilingualStringComparator),
        },
        {
            field: "slug",
            title: t("page.field.slug"),
        },
        actionsColumn(page => <>
            <EditStaticPageAction page={ page } />
            <DeleteStaticPageAction resource={ page } />
            <PreviewStaticPageAction page={ page } />
        </>)
    ];

    const PagePreview = ({ page }: { page: StaticPage }) =>
        <Box className={ spacing.vertical } p={ 3 }>
            <div>
                <Typography variant="subtitle2">Polski</Typography>
                <Typography variant="h2">{ page.title.pl }</Typography>
                <div dangerouslySetInnerHTML={{ __html: page.content.pl }} />
            </div>
            <div>
                <Typography variant="subtitle2">English</Typography>
                <Typography variant="h2">{ page.title.en }</Typography>
                <div dangerouslySetInnerHTML={{ __html: page.content.en }} />
            </div>
        </Box>

    return <Page>
        <Page.Header maxWidth="lg">
            <Management.Breadcrumbs>
                <Typography color="textPrimary">{ t("page.index.title") }</Typography>
            </Management.Breadcrumbs>
            <Page.Title>{ t("page.index.title") }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <CreateStaticPageAction />
                <Button onClick={ updatePageList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            { selected.length > 0 && <BulkActions>
                <DeleteStaticPageAction resource={ selected }>
                    { action => <Button startIcon={ <Delete /> } onClick={ action }>{ t("actions.delete") }</Button> }
                </DeleteStaticPageAction>
            </BulkActions> }
            <Async async={ result } keepValue>{
                pages => <MaterialTable
                    title={ <MaterialTableTitle result={ result } label={ t("page.index.title") }/> }
                    columns={ columns }
                    data={ pages }
                    detailPanel={ page => <PagePreview page={ page } /> }
                    onSelectionChange={ pages => setSelected(pages) }
                    options={{ selection: true }}
                />
            }</Async>
        </Container>
    </Page>
}

export default StaticPageManagement;
